import argparse
import os
import json
from PIL import Image, ImageDraw
import numpy as np

colors = [(0,255,0), (255,0,0), (0,0,255), (255,0,255), (0,255,255), (255,255,0), (200,200,200), (127,127,127)]

from config import SEQ_SRC, RESULT_SRC

parser = argparse.ArgumentParser(description='Draw bboxs from results')
parser.add_argument('--path-res', type=str, help='path of result files', default=RESULT_SRC.format('OPE'))
parser.add_argument('--path-seq', type=str, help='path of sequence files', default=SEQ_SRC)
parser.add_argument('--benchmark', type=str, help='show results of benchmark', default='cvpr13')
parser.add_argument('--show-all-res', help='show all result files', default=False, action='store_true')
parser.add_argument('--stride', type=int, help='set sample how many imgs to draw bboxs', default=1)
parser.add_argument('--path-bbox-res', type=str, help='set path to save output results', default='./res_bboxs/')
parser.add_argument('--no-text', help='no text for bounding box', default=False, action='store_true')
parser.add_argument('--bbox-color', type=str, help='color of bbox', default=None)

args = parser.parse_args()

if args.bbox_color is not None:
    colors = [args.bbox_color] * 10

path_results = os.listdir(args.path_res)
path_results_full = list(map(lambda x: os.path.join(args.path_res, x), path_results))

sort_by_time = lambda x: -os.path.getmtime(x)

path_results_full.sort(key=sort_by_time)

if args.show_all_res:
    num_show_res = len(path_results_full)
else:
    num_show_res = min(10, len(path_results_full))
path_results_full = path_results_full[0:num_show_res][::-1]
for i in range(num_show_res):
    print (i, ':', path_results_full[i])
arr_res = input('Input #reaults to draw, split by space : ')
arr_res = list(map(int, arr_res.split()))

fn_sq_list = os.path.join(args.path_seq, '{}.txt'.format(args.benchmark.lower()))
if os.path.exists(fn_sq_list):
    with open(fn_sq_list) as fs_benchmark:
        all_sq = fs_benchmark.readlines()
        all_sq = list(map(lambda x: x.strip().split('\t')[0], all_sq))
else:
    all_sq = [args.benchmark]

for i_sq in range(len(all_sq)):
    sq_name = all_sq[i_sq]
    path_sq_img = os.path.join(args.path_seq, sq_name)
    with open(os.path.join(path_sq_img, 'cfg.json')) as fs_cfg_sq:
        cfg_sq = json.load(fs_cfg_sq)
    st_frm = cfg_sq['startFrame']
    en_frm = cfg_sq['endFrame']
    nz = cfg_sq['nz']
    ext = cfg_sq['ext']
    with open(os.path.join(path_sq_img, 'groundtruth_rect.txt')) as fs_gt:
        gt_all = fs_gt.readlines()
    gt_all = list(map(lambda x: x.strip().split(), gt_all))
    if len(gt_all[0]) == 1:
        gt_all = list(map(lambda x: x[0].strip().split(','), gt_all))
    for i_gt, i_img in enumerate(range(st_frm, en_frm+1)):
        if (i_gt % args.stride) != 0:
            continue
        gt = np.array(list(map(float, gt_all[i_gt])))
        img_name = '{}.{}'.format(str(i_img).zfill(nz), ext)
        img_path = os.path.join(path_sq_img, 'img', img_name)
        img = Image.open(img_path).convert('RGB')
        draw = ImageDraw.Draw(img)
        gt[2:4] = gt[0:2]+gt[2:4]
        rect = tuple(gt)
        draw.rectangle(rect, fill=None, outline=colors[0])
        if not args.no_text:
            draw.text((0,0), 'ground truth', fill=colors[0])
        for i_color, i_trk in enumerate(arr_res):
            path_trk_res = path_results_full[i_trk]
            fn_res = os.path.join(path_trk_res, '{}.json'.format(sq_name))
            with open(fn_res) as fs_res:
                res = json.load(fs_res)
                rect = np.array(res[0]['res'][i_gt])
                rect[2:4] = rect[0:2]+rect[2:4]
                rect = tuple(rect)
                # print rect
                draw.rectangle(rect, fill=None, outline=colors[i_color+1])
                if not args.no_text:
                    draw.text((0,20*(i_color+1)), path_trk_res, fill=colors[i_color+1])
        img_path_dst = os.path.join(args.path_bbox_res, sq_name)
        if not os.path.exists(img_path_dst):
            os.makedirs(img_path_dst)
        fn_img_path_dst = os.path.join(img_path_dst, img_name)
        print (fn_img_path_dst)
        img.save(fn_img_path_dst)
        # print img_path, gt


        # for i_trk in xrange(len(arr_res)):
print (all_sq[0])


