import getopt

import numpy as np
from PIL import Image
from config import *
from scripts import *

import logging
logging.getLogger().setLevel(logging.INFO)

def main(argv):
    if not osp.exists(TRACKER_SRC):
        os.makedirs(TRACKER_SRC)
    trackers = os.listdir(TRACKER_SRC)
    evalTypes = ['OPE', 'SRE', 'TRE']
    loadSeqs = 'TB50'
    seqs = []
    try:
        opts, args = getopt.getopt(argv, "ht:e:s:",["tracker=","evaltype="
            ,"sequence="])
    except getopt.GetoptError:
        logging.error('usage : run_trackers.py -t <trackers> -s <sequences> ' \
            + '-e <evaltypes>')
        sys.exit(1)

    for opt, arg in opts:
        if opt == '-h':
            logging.info('usage : run_trackers.py -t <trackers> -s <sequences> ' \
                + '-e <evaltypes>')
            sys.exit(0)
        elif opt in ("-t", "--tracker"):
            trackers = [x.strip() for x in arg.split(',')]
            # trackers = [arg]
        elif opt in ("-s", "--sequence"):
            loadSeqs = arg
            if loadSeqs != 'All' and loadSeqs != 'all' and \
                loadSeqs != 'tb50' and loadSeqs != 'tb100' and \
                loadSeqs != 'cvpr13':
                loadSeqs = [x.strip() for x in arg.split(',')]
        elif opt in ("-e", "--evaltype"):
            evalTypes = [x.strip() for x in arg.split(',')]
            # evalTypes = [arg]

    if SETUP_SEQ:
        logging.info('Setup sequences ...')
        butil.setup_seqs(loadSeqs)
    testname = loadSeqs
    logging.info('Starting benchmark for {0} trackers, evalTypes : {1}'.format(
        len(trackers), evalTypes))
    for evalType in evalTypes:
        seqNames = butil.get_seq_names(loadSeqs)
        seqs = butil.load_seq_configs(seqNames)
        trackerResults = run_trackers(
            trackers, seqs, evalType, shiftTypeSet)
        for tracker in trackers:
            results = trackerResults[tracker]
            if len(results) > 0:
                evalResults, attrList = butil.calc_result(tracker,
                    seqs, results, evalType)
                all_err = []
                for seq in seqs:
                    n_success_frm = 0
                    n_fail_frm = 0
                    for ec in seq.errCenter:
                        if ec < 20.0:
                            n_success_frm += 1
                        else:
                            n_fail_frm += 1
                    err = n_success_frm/float(n_success_frm+n_fail_frm)
                    all_err.append(err)

                logging.info("Result of Sequences\t -- '{0}'".format(tracker))
                for i_seq, seq in enumerate(seqs):
                    try:
                        logging.info('\t\'{0}\'{1}'.format(seq.name, " "*(12 - len(seq.name))) 
                            + "\taveCoverage : {0:.3f}".format(sum(seq.aveCoverage)/len(seq.aveCoverage)) 
                            + "\taveErrCenter : {0:.3f}".format(sum(seq.aveErrCenter)/len(seq.aveErrCenter))
                            + "\tprec@20 : {0:.3f}".format(all_err[i_seq]))
                    except:
                        logging.error('\t\'{0}\'  ERROR!!'.format(seq.name))

                avg_err = sum(all_err) / len(all_err)
                logging.info('Precision plot score for ALL: {}'.format(round(avg_err,3)))
                logging.info("Result of attributes\t -- '{0}'".format(tracker))
                for attr in attrList:
                    logging.info("\t\'{0}\'".format(attr.name)
                        + "\toverlap : {0:02.1f}%".format(attr.overlap)
                        + "\tfailures : {0:.1f}".format(attr.error)
                        + "\tAUC : {0:.3f}".format(sum(attr.successRateList) / len(attr.successRateList)))

                if SAVE_RESULT : 
                    butil.save_scores(attrList, testname)

def run_trackers(trackers, seqs, evalType, shiftTypeSet):
    tmpRes_path = RESULT_SRC.format('tmp/{0}/'.format(evalType))
    if not os.path.exists(tmpRes_path):
        os.makedirs(tmpRes_path)

    numSeq = len(seqs)
    numTrk = len(trackers)

    trackerResults = dict((t,list()) for t in trackers)
    for idxSeq in range(numSeq):
        s = seqs[idxSeq]
        
        subSeqs, subAnno = butil.get_sub_seqs(s, 20.0, evalType)

        for idxTrk in range(len(trackers)):         
            t = trackers[idxTrk]
            full_path_t = os.path.join(TRACKER_SRC, t)
            if not os.path.exists(full_path_t):
                os.makedirs(full_path_t)
                logging.info('{0} does not exists, make dir for it.'.format(t))
            if not OVERWRITE_RESULT:
                trk_src = os.path.join(RESULT_SRC.format(evalType), t)
                result_src = os.path.join(trk_src, s.name+'.json')
                if os.path.exists(result_src):
                    seqResults = butil.load_seq_result(evalType, t, s.name)
                    trackerResults[t].append(seqResults)
                    continue
            seqResults = []
            seqLen = len(subSeqs)
            for idx in range(seqLen):
                logging.info('{0}_{1}, {2}_{3}:{4}/{5} - {6}'.format(
                    idxTrk + 1, t, idxSeq + 1, s.name, idx + 1, seqLen, \
                    evalType))
                rp = tmpRes_path + '_' + t + '_' + str(idx+1) + '/'
                if SAVE_IMAGE and not os.path.exists(rp):
                    os.makedirs(rp)
                subS = subSeqs[idx]
                subS.name = s.name + '_' + str(idx)
                    
                funcName = 'run_{0}(subS, rp, SAVE_IMAGE)'.format(t)
                res = eval(funcName)

                if evalType == 'SRE':
                    r = Result(t, s.name, subS.startFrame, subS.endFrame,
                        res['type'], evalType, res['res'], res['fps'], shiftTypeSet[idx])
                else:
                    r = Result(t, s.name, subS.startFrame, subS.endFrame,
                        res['type'], evalType, res['res'], res['fps'], None)
                try: r.tmplsize = butil.d_to_f(res['tmplsize'][0])
                except: pass
                r.refresh_dict()
                seqResults.append(r)
            #end for subseqs
            if SAVE_RESULT:
                butil.save_seq_result(seqResults)

            trackerResults[t].append(seqResults)
        #end for tracker
    #end for allseqs
    return trackerResults

if __name__ == "__main__":
    main(sys.argv[1:])
